@@ -244,7 +244,7 @@ var Artemis;
                 {name: "Large", visible: true},
                 {name: "Persistent Size", visible: true},
                 {name: "User ID", visible: true},
-                {name: "Validated User", visible: true},
+                {name: "Validated User", visible: false},
                 {name: "Original Queue (Expiry/DLQ's only)", visible: true}
            ]
         };
@@ -303,7 +303,7 @@ var Artemis;
                 itemField: 'expiration',
                 header: 'Expires',
                 templateFn: function(value) {
-                    return formatExpires(value);
+                    return formatExpires(value, false);
                 }
             },
             {
@@ -326,7 +326,7 @@ var Artemis;
                 header: 'User ID'
             },
             {
-                itemField: 'StringProperties',
+                itemField: 'validatedUser',
                 header: 'Validated User',
                 templateFn: function(value) {
                     return value._AMQ_VALIDATED_USER;
@@ -464,8 +464,8 @@ var Artemis;
             return (value < 10 ? '0' : '') + value;
         }
 
-        function formatExpires(timestamp) {
-             if (isNaN(timestamp)) {
+        function formatExpires(timestamp, addTimestamp) {
+             if (isNaN(timestamp) || typeof timestamp !== "number") {
                 return timestamp;
              }
              if (timestamp == 0) {
@@ -477,20 +477,29 @@ var Artemis;
                 var hours = pad2(Math.floor((duration / MS_PER_HOUR) % 24));
                 var mins  = pad2(Math.floor((duration / MS_PER_MIN) % 60));
                 var secs  = pad2(Math.floor((duration / MS_PER_SEC) % 60));
+                var ret;
                 if (expiresIn < 0) {
                    // "HH:mm:ss ago"
-                   return hours + ":" + mins + ":" + secs + " ago";
+                   ret = hours + ":" + mins + ":" + secs + " ago";
+                } else {
+                   // "in HH:mm:ss"
+                   ret = "in " + hours + ":" + mins + ":" + secs;
                 }
-                // "in HH:mm:ss ago"
-                return "in " + hours + ":" + mins + ":" + secs;
+                if (addTimestamp) {
+                   ret += ", at " + formatTimestamp(timestamp);
+                }
+                return ret;
              }
              return formatTimestamp(timestamp);
           }
 
           function formatTimestamp(timestamp) {
-             if (isNaN(timestamp)) {
+             if (isNaN(timestamp) || typeof timestamp !== "number") {
                 return timestamp;
              }
+             if (timestamp === 0) {
+                return "N/A";
+             }
              var d = new Date(timestamp);
              // "yyyy-MM-dd HH:mm:ss"
              //add 1 to month as getmonth returns the position not the actual month
@@ -499,12 +508,20 @@ var Artemis;
 
         var typeLabels = ["default", "1", "object", "text", "bytes", "map", "stream", "embedded"];
         function formatType(type) {
-            if (isNaN(type)) {
+            if (isNaN(type) || typeof type !== "number") {
                 return type;
             }
             return type > -1 && type < 8 ? typeLabels[type] : type
         }
 
+        var bindingTypeLabels = ["local-queue", "remote-queue", "divert"];
+        function formatBindingType(type) {
+            if (isNaN(type) || typeof type !== "number") {
+                return type;
+            }
+            return type > -1 && type < 3 ? bindingTypeLabels[type] : type
+        }
+
         ctrl.refresh = function() {
             Artemis.log.debug(ctrl.filter)
             //if refreshing always return to the first page
@@ -518,7 +535,7 @@ var Artemis;
         }
 
         function formatPersistentSize(bytes) {
-            if(isNaN(bytes) || bytes < 0) return "n/a";
+            if(isNaN(bytes) || typeof bytes !== "number" || bytes < 0) return "N/A";
             if(bytes < 10240) return bytes.toLocaleString() + " Bytes";
             if(bytes < 1048576) return (bytes / 1024).toFixed(2) + " KB";
             if(bytes < 1073741824) return (bytes / 1048576).toFixed(2) + " MB";
@@ -793,7 +810,7 @@ var Artemis;
             angular.forEach(message, function (value, key) {
                 if (!_.some(ignoreColumns, function (k) { return k === key; }) && !_.some(flattenColumns, function (k) { return k === key; })) {
                     if(key === "expiration") {
-                        value += " (" + formatExpires(value) + ")";
+                        value += " (" + formatExpires(value, true) + ")";
                     } else if(key === "persistentSize") {
                         value += " (" + formatPersistentSize(value) + ")";
                     } else if(key === "timestamp") {
@@ -809,15 +826,23 @@ var Artemis;
 
 
         var amqpEncodingLabels = [
-			"amqp-unknown", "amqp-null", "amqp-data", "amqp-sequence", "amqp-value-null",
-			"amqp-value-string", "amqp-value-binary", "amqp-value-map", "amqp-value-list"];
+            "amqp-unknown", "amqp-null", "amqp-data", "amqp-sequence", "amqp-value-null",
+            "amqp-value-string", "amqp-value-binary", "amqp-value-map", "amqp-value-list"];
         function formatAmqpEncoding(enc) {
-            if (isNaN(enc)) {
+            if (isNaN(enc) || typeof enc !== "number") {
                 return enc;
             }
             return enc > -1 && enc < 9 ? amqpEncodingLabels[enc] : enc;
         }
 
+        var routingTypes = ["multicast", "anycast"];
+        function formatRoutingType(rt) {
+            if (isNaN(rt) || typeof rt !== "number") {
+                return enc;
+            }
+            return rt > -1 && rt < 2 ? routingTypes[rt] : rt;
+        }
+
         function createProperties(message) {
             var properties = [];
             angular.forEach(message, function (value, key) {
@@ -825,9 +850,27 @@ var Artemis;
                     Artemis.log.debug("key=" + key + " value=" + value);
                     angular.forEach(value, function (v2, k2) {
                     Artemis.log.debug("key=" + k2 + " value=" + v2);
-						if(k2 === "JMS_AMQP_ORIGINAL_ENCODING") {
-							v2 += " (" + formatAmqpEncoding(v2) + ")";
-						}
+                        if(k2 === "_AMQ_Binding_Type") {
+                            v2 += " (" + formatBindingType(v2) + ")";
+                        } else if(k2 === "JMS_AMQP_ORIGINAL_ENCODING") {
+                            v2 += " (" + formatAmqpEncoding(v2) + ")";
+                        } else if(k2 === "_AMQ_ACTUAL_EXPIRY") {
+                            v2 += " (" + formatTimestamp(v2) + ")";
+                        } else if(k2 === "_AMQ_NotifTimestamp") {
+                            v2 += " (" + formatTimestamp(v2) + ")";
+                        } else if(k2 === "_AMQ_ROUTING_TYPE") {
+                            v2 += " (" + formatRoutingType(v2) + ")";
+                        } else if(k2 === "_AMQ_ORIG_ROUTING_TYPE") {
+                            v2 += " (" + formatRoutingType(v2) + ")";
+                        } else if(k2 === "extraProperties._AMQ_ACTUAL_EXPIRY") {
+                            v2 += " (" + formatTimestamp(v2) + ")";
+                        } else if(k2 === "messageAnnotations.x-opt-ACTUAL-EXPIRY") {
+                            v2 += " (" + formatTimestamp(v2) + ")";
+                        } else if(k2 === "properties.creationTime") {
+                            v2 += " (" + formatTimestamp(v2) + ")";
+                        } else if(k2 === "__HDR_BROKER_IN_TIME") {
+                            v2 += " (" + formatTimestamp(v2) + ")";
+                        }
                         properties.push({key: k2, value: v2});
                     });
                 }
