@@ -39,7 +39,6 @@ import io.netty.handler.codec.mqtt.MqttSubscribeMessage;
 import io.netty.handler.codec.mqtt.MqttUnsubAckMessage;
 import io.netty.handler.codec.mqtt.MqttUnsubAckPayload;
 import io.netty.handler.codec.mqtt.MqttUnsubscribeMessage;
-import io.netty.handler.codec.mqtt.MqttVersion;
 import io.netty.util.CharsetUtil;
 import io.netty.util.ReferenceCountUtil;
 import org.apache.activemq.artemis.api.core.ActiveMQSecurityException;
@@ -52,6 +51,7 @@ import org.jboss.logging.Logger;
 
 import static io.netty.handler.codec.mqtt.MqttProperties.MqttPropertyType.AUTHENTICATION_DATA;
 import static io.netty.handler.codec.mqtt.MqttProperties.MqttPropertyType.AUTHENTICATION_METHOD;
+import static io.netty.handler.codec.mqtt.MqttProperties.MqttPropertyType.SESSION_EXPIRY_INTERVAL;
 
 /**
  * This class is responsible for receiving and sending MQTT packets, delegating behaviour to one of the
@@ -129,7 +129,7 @@ public class MQTTProtocolHandler extends ChannelInboundHandlerAdapter {
          AuditLogger.setRemoteAddress(connection.getRemoteAddress());
       }
 
-      MQTTUtil.logMessage(session.getState(), message, true);
+      MQTTUtil.logMessage(session.getState(), message, true, session.getVersion());
 
       if (this.ctx == null) {
          this.ctx = ctx;
@@ -174,7 +174,7 @@ public class MQTTProtocolHandler extends ChannelInboundHandlerAdapter {
                handleUnsubscribe((MqttUnsubscribeMessage) message);
                break;
             case DISCONNECT:
-               disconnect(false);
+               disconnect(false, message);
                break;
             case UNSUBACK:
             case SUBACK:
@@ -223,6 +223,7 @@ public class MQTTProtocolHandler extends ChannelInboundHandlerAdapter {
    }
 
    void handleConnect(MqttConnectMessage connect) throws Exception {
+      session.setVersion(MQTTVersion.getVersion(connect.variableHeader().version()));
       /*
        * Perform authentication *before* attempting redirection because redirection may be based on the user's role.
        */
@@ -243,12 +244,11 @@ public class MQTTProtocolHandler extends ChannelInboundHandlerAdapter {
 
       if (connection.getTransportConnection().getRouter() == null || !protocolManager.getRoutingHandler().route(connection, session, connect)) {
          /* [MQTT-3.1.2-2] Reject unsupported clients. */
-         int packetVersion = connect.variableHeader().version();
-         if (packetVersion != MqttVersion.MQTT_3_1.protocolLevel() &&
-            packetVersion != MqttVersion.MQTT_3_1_1.protocolLevel() &&
-            packetVersion != MqttVersion.MQTT_5.protocolLevel()) {
+         if (session.getVersion() != MQTTVersion.MQTT_3_1 &&
+            session.getVersion() != MQTTVersion.MQTT_3_1_1 &&
+            session.getVersion() != MQTTVersion.MQTT_5) {
 
-            if (packetVersion <= MqttVersion.MQTT_3_1_1.protocolLevel()) {
+            if (session.getVersion().getVersion() <= MQTTVersion.MQTT_3_1_1.getVersion()) {
                // See MQTT-3.1.2-2 at http://docs.oasis-open.org/mqtt/mqtt/v3.1.1/os/mqtt-v3.1.1-os.html#_Toc398718030
                sendConnack(MQTTReasonCodes.UNACCEPTABLE_PROTOCOL_VERSION_3);
             } else {
@@ -281,6 +281,16 @@ public class MQTTProtocolHandler extends ChannelInboundHandlerAdapter {
    }
 
    void disconnect(boolean error) {
+      disconnect(error, null);
+   }
+
+   void disconnect(boolean error, MqttMessage disconnect) {
+      if (disconnect != null && disconnect.variableHeader() instanceof MqttReasonCodeAndPropertiesVariableHeader) {
+         Integer sessionExpiryInterval = MQTTUtil.getProperty(Integer.class, ((MqttReasonCodeAndPropertiesVariableHeader)disconnect.variableHeader()).properties(), SESSION_EXPIRY_INTERVAL, null);
+         if (sessionExpiryInterval != null) {
+            session.getState().setClientSessionExpiryInterval(sessionExpiryInterval);
+         }
+      }
       session.getConnectionManager().disconnect(error);
    }
 
@@ -408,7 +418,7 @@ public class MQTTProtocolHandler extends ChannelInboundHandlerAdapter {
       if (this.protocolManager.invokeOutgoing(message, connection) != null) {
          return;
       }
-      MQTTUtil.logMessage(session.getState(), message, false);
+      MQTTUtil.logMessage(session.getState(), message, false, session.getVersion());
       ctx.writeAndFlush(message, ctx.voidPromise());
    }
 
