@@ -41,12 +41,12 @@ import org.apache.activemq.artemis.core.protocol.openwire.OpenWireProtocolManage
 import org.apache.activemq.artemis.core.protocol.openwire.util.OpenWireUtil;
 import org.apache.activemq.artemis.core.server.ActiveMQServer;
 import org.apache.activemq.artemis.core.server.ActiveMQServerLogger;
-import org.apache.activemq.artemis.core.server.BindingQueryResult;
 import org.apache.activemq.artemis.core.server.MessageReference;
 import org.apache.activemq.artemis.core.server.QueueQueryResult;
 import org.apache.activemq.artemis.core.server.ServerConsumer;
 import org.apache.activemq.artemis.core.server.ServerSession;
 import org.apache.activemq.artemis.core.server.SlowConsumerDetectionListener;
+import org.apache.activemq.artemis.core.server.impl.AddressInfo;
 import org.apache.activemq.artemis.core.settings.impl.AddressSettings;
 import org.apache.activemq.artemis.reader.MessageUtil;
 import org.apache.activemq.artemis.spi.core.protocol.SessionCallback;
@@ -200,8 +200,8 @@ public class AMQSession implements SessionCallback {
       //lazy allocation of the cache
       if (existingQueuesCache == null) {
          //16 means 64 bytes with 32 bit references or 128 bytes with 64 bit references -> 1 or 2 cache lines with common archs
-         existingQueuesCache = new String[16];
-         assert (Integer.bitCount(existingQueuesCache.length) == 1) : "existingQueuesCache.length must be power of 2";
+         existingQueuesCache = new String[protocolManager.getOpenWireDestinationCacheSize()];
+         assert (Integer.bitCount(existingQueuesCache.length) == 1) : "openWireDestinationCacheSize must be a power of 2";
          this.existingQueuesCache = existingQueuesCache;
       }
       final int hashCode = physicalName.hashCode();
@@ -225,23 +225,23 @@ public class AMQSession implements SessionCallback {
    }
 
    private boolean checkAutoCreateQueue(SimpleString queueName, boolean isTemporary, String filter) throws Exception {
+
       boolean hasQueue = true;
       if (!connection.containsKnownDestination(queueName)) {
-
-         BindingQueryResult bindingQuery = server.bindingQuery(queueName);
-         QueueQueryResult queueBinding = server.queueQuery(queueName);
+         QueueQueryResult queueQuery = server.queueQuery(queueName);
 
          try {
-            if (!queueBinding.isExists()) {
-               if (bindingQuery.isAutoCreateQueues()) {
+            if (!queueQuery.isExists()) {
+               if (queueQuery.isAutoCreateQueues()) {
                   SimpleString queueNameToUse = queueName;
                   SimpleString addressToUse = queueName;
                   RoutingType routingTypeToUse = RoutingType.ANYCAST;
                   if (CompositeAddress.isFullyQualified(queueName.toString())) {
                      addressToUse = CompositeAddress.extractAddressName(queueName);
                      queueNameToUse = CompositeAddress.extractQueueName(queueName);
-                     if (bindingQuery.getAddressInfo() != null) {
-                        routingTypeToUse = bindingQuery.getAddressInfo().getRoutingType();
+                     AddressInfo addressInfo = server.getAddressInfo(addressToUse);
+                     if (addressInfo != null) {
+                        routingTypeToUse = addressInfo.getRoutingType();
                      } else {
                         AddressSettings as = server.getAddressSettingsRepository().getMatch(addressToUse.toString());
                         routingTypeToUse = as.getDefaultAddressRoutingType();
@@ -257,7 +257,6 @@ public class AMQSession implements SessionCallback {
             // In case another thread created the queue before us but after we did the binding query
             hasQueue = true;
          }
-
       }
       return hasQueue;
    }
