@@ -128,7 +128,7 @@ public class SoakPagingTest extends SmokeTestBase {
       } else if (protocol.toUpperCase().equals("CORE") || protocol.toUpperCase().equals("ARTEMIS")) {
          return new org.apache.activemq.artemis.jms.client.ActiveMQConnectionFactory(uri);
       } else {
-         throw new IllegalStateException("Unkown:" + protocol);
+         throw new IllegalStateException("Unknown:" + protocol);
       }
    }
 
